==> bigloo2.0a.tar.gz: bigloo2.0a/runtime/Ieee/control5.scm <==
;*---------------------------------------------------------------------*/
;*   A pratical implementation for the Scheme programming language     */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*   Copyright (c) 1992-1999 Manuel Serrano                            */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo-request@kaolin.unice.fr                                */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/runtime/Ieee/control5.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Feb 27 14:11:26 1998                          */
;*    Last change :  Fri Mar 13 18:27:03 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    An implementation of the R5RS multiple values.                   */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __r5_control_features_6_4
    
   (import  (__error                   "Llib/error.scm"))
   
   (use     (__type                    "Llib/type.scm")
	    (__bigloo                  "Llib/bigloo.scm")
	    (__tvector                 "Llib/tvector.scm")
	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
	    (__r4_vectors_6_8          "Ieee/vector.scm")
	    (__r4_strings_6_7          "Ieee/string.scm")
	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
	    (__r4_booleans_6_1         "Ieee/boolean.scm")
	    (__r4_symbols_6_4          "Ieee/symbol.scm")
	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
	    (__r4_control_features_6_9 "Ieee/control.scm")

	    (__evenv                   "Eval/evenv.scm"))

   (export  (values           . args)
	    (call-with-values ::procedure ::procedure)
	    *res-number*
	    *res1*
	    *res2*
	    *res3*))

;*---------------------------------------------------------------------*/
;*    *values* ...                                                     */
;*---------------------------------------------------------------------*/
(define *res-number* 1)
(define *res1* #unspecified)
(define *res2* #unspecified)
(define *res3* #unspecified)

;*---------------------------------------------------------------------*/
;*    values ...                                                       */
;*    -------------------------------------------------------------    */
;*    Values with exactly one argument are not boxed.                  */
;*---------------------------------------------------------------------*/
(define (values . args)
   (let ((all-args args))
      (if (null? args)
	  (set! *res-number* 0)
	  (if (null? (cdr args))
		  (begin
		     (set! *res-number* 1)
		     (car args))
		  (let ((res0 (car args)))
		     (set! args (cdr args))
		     (set! *res1* (car args))
		     (set! args (cdr args))
		     (if (pair? args)
			 (begin
			    (set! *res2* (car args))
			    (set! args (cdr args))
			    (if (pair? args)
				(if (pair? (cdr args))
				    (begin
				       (set! *res-number* -1)
				       all-args)
				    (begin
				       (set! *res3* (car args))
				       (set! *res-number* 4)
				       res0))
				(begin
				   (set! *res-number* 3)
				   res0)))
			 (begin
			    (set! *res-number* 2)
			    res0)))))))
	      
;*---------------------------------------------------------------------*/
;*    call-with-values ...                                             */
;*---------------------------------------------------------------------*/
(define (call-with-values producer consumer)
   (set! *res-number* 1)
   (let ((res0 (producer)))
      (case *res-number*
	 ((-1)
	  (apply consumer res0))
	 ((0)
	  (consumer))
	 ((1)
	  (consumer res0))
	 ((2)
	  (consumer res0 *res1*))
	 ((3)
	  (consumer res0 *res1* *res2*))
	 ((4)
	  (consumer res0 *res1* *res2* *res3*))
	 (else
	  (apply consumer res0)))))

==> bigloo_2.4b.orig.tar.gz: bigloo2.4b/runtime/Ieee/control5.scm <==
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/runtime/Ieee/control5.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Feb 27 14:11:26 1998                          */
;*    Last change :  Mon May  7 18:51:27 2001 (serrano)                */
;*    -------------------------------------------------------------    */
;*    An implementation of the R5RS multiple values.                   */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __r5_control_features_6_4
    
   (import  __error)
   
   (use     __type
	    __bigloo
	    __tvector
	    __r4_equivalence_6_2
	    __r4_vectors_6_8
	    __r4_strings_6_7
	    __r4_numbers_6_5_fixnum
	    __r4_booleans_6_1
	    __r4_symbols_6_4
	    __r4_pairs_and_lists_6_3
	    __r4_control_features_6_9

	    __evenv)

   (export  (values           . args)
	    (call-with-values ::procedure ::procedure)
	    *res-number*
	    *res1*
	    *res2*
	    *res3*))

;*---------------------------------------------------------------------*/
;*    *values* ...                                                     */
;*---------------------------------------------------------------------*/
(define *res-number* 1)
(define *res1* #unspecified)
(define *res2* #unspecified)
(define *res3* #unspecified)

;*---------------------------------------------------------------------*/
;*    values ...                                                       */
;*    -------------------------------------------------------------    */
;*    Values with exactly one argument are not boxed.                  */
;*---------------------------------------------------------------------*/
(define (values . args)
   (let ((all-args args))
      (if (null? args)
	  (set! *res-number* 0)
	  (if (null? (cdr args))
		  (begin
		     (set! *res-number* 1)
		     (car args))
		  (let ((res0 (car args)))
		     (set! args (cdr args))
		     (set! *res1* (car args))
		     (set! args (cdr args))
		     (if (pair? args)
			 (begin
			    (set! *res2* (car args))
			    (set! args (cdr args))
			    (if (pair? args)
				(if (pair? (cdr args))
				    (begin
				       (set! *res-number* -1)
				       all-args)
				    (begin
				       (set! *res3* (car args))
				       (set! *res-number* 4)
				       res0))
				(begin
				   (set! *res-number* 3)
				   res0)))
			 (begin
			    (set! *res-number* 2)
			    res0)))))))
	      
;*---------------------------------------------------------------------*/
;*    call-with-values ...                                             */
;*---------------------------------------------------------------------*/
(define (call-with-values producer consumer)
   (set! *res-number* 1)
   (let ((res0 (producer)))
      (case *res-number*
	 ((-1)
	  (apply consumer res0))
	 ((0)
	  (consumer))
	 ((1)
	  (consumer res0))
	 ((2)
	  (consumer res0 *res1*))
	 ((3)
	  (consumer res0 *res1* *res2*))
	 ((4)
	  (consumer res0 *res1* *res2* *res3*))
	 (else
	  (apply consumer res0)))))
